@@ -31,7 +31,7 @@ void PCEFast_PSG::SetVolume(double new_volume)
    Blip_Synth_set_volume(&Synth, OutputVolume / 6, 8192);
 }
 
-void PCEFast_PSG::SetChannelUserVolume(int chnum, uint8 new_volume)
+void PCEFast_PSG::SetChannelUserVolume(int chnum, int32 new_volume)
 {
     if(chnum >=6 || new_volume > 100) return;  // check valid args
     psg_channel *ch = &channel[chnum];
@@ -41,13 +41,24 @@ void PCEFast_PSG::SetChannelUserVolume(int chnum, uint8 new_volume)
 void PCEFast_PSG::UpdateOutput_Norm(const int32 timestamp, psg_channel *ch)
 {
    int32 samp[2];
+   int delta0;
+   int delta1;   
    int sv = ch->dda;
 
    samp[0] = dbtable[ch->vl[0]][sv];
    samp[1] = dbtable[ch->vl[1]][sv];
 
-   Blip_Synth_offset(&Synth, timestamp, (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100, sbuf[0]);
-   Blip_Synth_offset(&Synth, timestamp, (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100, sbuf[1]);
+   if(ch->user_volume < 100)
+   {
+       delta0 = (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100;
+       delta1 = (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100;
+   } else {
+       delta0 = samp[0] - ch->blip_prev_samp[0];
+       delta1 = samp[1] - ch->blip_prev_samp[1];
+   }
+    
+   Blip_Synth_offset(&Synth, timestamp, delta0, sbuf[0]);
+   Blip_Synth_offset(&Synth, timestamp, delta1, sbuf[1]);
 
    ch->blip_prev_samp[0] = samp[0];
    ch->blip_prev_samp[1] = samp[1];
@@ -56,13 +67,24 @@ void PCEFast_PSG::UpdateOutput_Norm(const int32 timestamp, psg_channel *ch)
 void PCEFast_PSG::UpdateOutput_Noise(const int32 timestamp, psg_channel *ch)
 {
    int32 samp[2];
+   int delta0;
+   int delta1;
    int sv = ((ch->lfsr & 1) << 5) - (ch->lfsr & 1); //(ch->lfsr & 0x1) ? 0x1F : 0;
 
    samp[0] = dbtable[ch->vl[0]][sv];
    samp[1] = dbtable[ch->vl[1]][sv];
 
-   Blip_Synth_offset(&Synth, timestamp, (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100, sbuf[0]);
-   Blip_Synth_offset(&Synth, timestamp, (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100, sbuf[1]);
+   if(ch->user_volume < 100)
+   {
+       delta0 = (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100;
+       delta1 = (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100;
+   } else {
+       delta0 = samp[0] - ch->blip_prev_samp[0];
+       delta1 = samp[1] - ch->blip_prev_samp[1];
+   }
+    
+   Blip_Synth_offset(&Synth, timestamp, delta0, sbuf[0]);
+   Blip_Synth_offset(&Synth, timestamp, delta1, sbuf[1]);
 
    ch->blip_prev_samp[0] = samp[0];
    ch->blip_prev_samp[1] = samp[1];
@@ -71,11 +93,22 @@ void PCEFast_PSG::UpdateOutput_Noise(const int32 timestamp, psg_channel *ch)
 void PCEFast_PSG::UpdateOutput_Off(const int32 timestamp, psg_channel *ch)
 {
    int32 samp[2];
+   int delta0;
+   int delta1;
 
    samp[0] = samp[1] = 0;
 
-   Blip_Synth_offset(&Synth, timestamp, (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100, sbuf[0]);
-   Blip_Synth_offset(&Synth, timestamp, (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100, sbuf[1]);
+   if(ch->user_volume < 100)
+   {
+       delta0 = (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100;
+       delta1 = (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100;
+   } else {
+       delta0 = samp[0] - ch->blip_prev_samp[0];
+       delta1 = samp[1] - ch->blip_prev_samp[1];
+   }
+    
+   Blip_Synth_offset(&Synth, timestamp, delta0, sbuf[0]);
+   Blip_Synth_offset(&Synth, timestamp, delta1, sbuf[1]);
 
    ch->blip_prev_samp[0] = samp[0];
    ch->blip_prev_samp[1] = samp[1];
@@ -85,12 +118,23 @@ void PCEFast_PSG::UpdateOutput_Off(const int32 timestamp, psg_channel *ch)
 void PCEFast_PSG::UpdateOutput_Accum(const int32 timestamp, psg_channel *ch)
 {
    int32 samp[2];
+   int delta0;
+   int delta1;
 
    samp[0] = ((int32)dbtable_volonly[ch->vl[0]] * ((int32)ch->samp_accum - 496)) >> (8 + 5);
    samp[1] = ((int32)dbtable_volonly[ch->vl[1]] * ((int32)ch->samp_accum - 496)) >> (8 + 5);
 
-   Blip_Synth_offset(&Synth, timestamp, (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100, sbuf[0]);
-   Blip_Synth_offset(&Synth, timestamp, (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100, sbuf[1]);
+   if(ch->user_volume < 100)
+   {
+       delta0 = (samp[0] - ch->blip_prev_samp[0]) * ch->user_volume / 100;
+       delta1 = (samp[1] - ch->blip_prev_samp[1]) * ch->user_volume / 100;
+   } else {
+       delta0 = samp[0] - ch->blip_prev_samp[0];
+       delta1 = samp[1] - ch->blip_prev_samp[1];
+   }
+    
+   Blip_Synth_offset(&Synth, timestamp, delta0, sbuf[0]);
+   Blip_Synth_offset(&Synth, timestamp, delta1, sbuf[1]);
 
    ch->blip_prev_samp[0] = samp[0];
    ch->blip_prev_samp[1] = samp[1];
